﻿using System;
using System.Transactions;

namespace HeuristicLab.Services.Hive.DataAccess {
  public class TransactionManager {
    public void UseTransaction(Action call, bool serializable = false) {
      TransactionScope transaction = serializable ? CreateSerializableTransaction() : CreateTransaction();
      try {
        call();
        transaction.Complete();
      }
      finally {
        transaction.Dispose();
      }
    }

    public T UseTransaction<T>(Func<T> call, bool serializable = false) {
      TransactionScope transaction = serializable ? CreateSerializableTransaction() : CreateTransaction();
      try {
        T result = call();
        transaction.Complete();
        return result;
      }
      finally {
        transaction.Dispose();
      }
    }

    private static TransactionScope CreateTransaction() {
      return new TransactionScope(TransactionScopeOption.Required, new TransactionOptions { IsolationLevel = IsolationLevel.ReadUncommitted });
    }
    private static TransactionScope CreateSerializableTransaction() {
      return new TransactionScope(TransactionScopeOption.Required, new TransactionOptions { IsolationLevel = IsolationLevel.Serializable });
    }
  }
}
